@@ -173,8 +173,18 @@ def main():
         batch_size = config.get('batch_size', 16)
         img_size = config.get('img_size', 640)
         output_formats = config.get('output_formats', ['onnx'])
+        training_mode = config.get('training_mode', 'fine_tune')  # NEW: "fine_tune" or "from_scratch"
+        resume_checkpoint = config.get('resume_checkpoint', None)  # Path to checkpoint for resuming
+        # Label management strategy
+        label_strategy = config.get('labelStrategy', 'merge_all')  # 'merge_all' or 'base_dataset'
+        base_dataset_index = config.get('baseDatasetIndex', None)  # Index of base dataset (0-based)
         
         log_progress(f"🔍 DEBUG: After extraction, epochs = {epochs}")
+        log_progress(f"🔍 DEBUG: training_mode = {training_mode}")
+        log_progress(f"🔍 DEBUG: label_strategy = {label_strategy}, base_dataset_index = {base_dataset_index}")
+        
+        if resume_checkpoint:
+            log_progress(f"🔄 Resume mode: Will continue training from checkpoint: {resume_checkpoint}")
         
         # Wandb configuration
         wandb_enabled = config.get('wandb_enabled', False)
@@ -210,6 +220,7 @@ def main():
                     img_size=img_size,
                     learning_rate=learning_rate,
                     patience=patience,
+                    training_mode=training_mode,
                     wandb_enabled=wandb_enabled)
         
         # Discover datasets using aegis-vision
@@ -230,11 +241,72 @@ def main():
         # Preprocess datasets (convert COCO standard to flat format)
         preprocessed_datasets = preprocess_datasets(datasets_found, WORKING_DIR)
         
+        # For base_dataset strategy, validate that we have the required information
+        actual_base_index = base_dataset_index
+        if label_strategy == 'base_dataset':
+            # Get the base dataset name from config if available
+            base_dataset_name = config.get('baseDatasetName', None)
+            datasets_config = config.get('datasets', [])
+            
+            log_progress(f"🔍 DEBUG: Label strategy is 'base_dataset', base_dataset_index = {base_dataset_index}")
+            log_progress(f"🔍 DEBUG: baseDatasetName = {base_dataset_name}")
+            log_progress(f"🔍 DEBUG: Found {len(datasets_found)} datasets, preprocessed {len(preprocessed_datasets)} datasets")
+            
+            # Validate that we have base dataset information
+            if actual_base_index is None:
+                error_msg = (
+                    f"❌ Configuration error: label_strategy='base_dataset' but baseDatasetIndex is None. "
+                    f"This indicates a bug in the UI or agent configuration. "
+                    f"baseDatasetName={base_dataset_name}, datasets_config length={len(datasets_config)}. "
+                    f"Training cannot proceed without knowing which dataset to use as the base."
+                )
+                log_progress(error_msg, status="error")
+                raise ValueError(error_msg)
+            
+            # Validate index is in range
+            if actual_base_index < 0 or actual_base_index >= len(preprocessed_datasets):
+                error_msg = (
+                    f"❌ Configuration error: baseDatasetIndex={actual_base_index} is out of range. "
+                    f"Valid range: 0-{len(preprocessed_datasets)-1}. "
+                    f"Found {len(preprocessed_datasets)} preprocessed datasets. "
+                    f"This may indicate a mismatch between the selected datasets and discovered datasets."
+                )
+                log_progress(error_msg, status="error")
+                raise ValueError(error_msg)
+            
+            # If we have an index, try to get the name for verification
+            if actual_base_index is not None and datasets_config and actual_base_index < len(datasets_config):
+                if not base_dataset_name:
+                    base_dataset_name = datasets_config[actual_base_index].get('name', '')
+                    log_progress(f"🔍 DEBUG: Got base dataset name from config: {base_dataset_name}")
+            
+            # Log all datasets for debugging
+            for i, (ds_path, ds_format) in enumerate(preprocessed_datasets):
+                marker = " <-- BASE" if i == actual_base_index else ""
+                log_progress(f"🔍 DEBUG: Preprocessed dataset {i}: {Path(ds_path).name} ({ds_format}){marker}")
+        
         # Merge and convert datasets to YOLO format
-        log_progress("⚡ Merging datasets and converting to YOLO format...")
+        if label_strategy == 'base_dataset' and actual_base_index is not None:
+            log_progress(f"⚡ Merging datasets with base dataset strategy (using dataset index {actual_base_index})...")
+        else:
+            log_progress("⚡ Merging datasets and converting to YOLO format...")
+            if label_strategy != 'merge_all':
+                log_progress(f"⚠️  Warning: label_strategy='{label_strategy}' but base_dataset_index is None, defaulting to merge_all")
         output_dir = WORKING_DIR / "yolo_dataset"
         merger = AdvancedCOCOtoYOLOMerger(output_dir=output_dir)
-        merge_result = merger.merge_and_convert(preprocessed_datasets)
+        
+        # Enable ASCII visualization for agent mode (helps verify bounding box handling)
+        # Can be disabled via config: "enable_ascii_visualization": false
+        enable_ascii_viz = config.get('enable_ascii_visualization', IS_AGENT)
+        if enable_ascii_viz:
+            log_progress("🎨 ASCII visualization enabled - will show 2 sample images per dataset with bounding boxes")
+        
+        merge_result = merger.merge_and_convert(
+            preprocessed_datasets,
+            enable_ascii_visualization=enable_ascii_viz,
+            label_strategy=label_strategy,
+            base_dataset_index=actual_base_index if label_strategy == 'base_dataset' else None
+        )
         
         dataset_yaml = merge_result['dataset_yaml']
         log_progress(f"✅ Dataset ready: {dataset_yaml}")
@@ -274,6 +346,8 @@ def main():
             weight_decay=weight_decay,
             warmup_epochs=warmup_epochs,
             patience=patience,
+            training_mode=training_mode,  # NEW: Pass training mode to control pretrained weights
+            resume_checkpoint=resume_checkpoint,  # NEW: Path to checkpoint for resuming training
             hsv_h=hsv_h,
             hsv_s=hsv_s,
             hsv_v=hsv_v,
